@@ -35,13 +35,6 @@ top-level ``unyt`` namespace:
 * :func:`unyt.dimensions.returns`
 """
 
-# -----------------------------------------------------------------------------
-# Copyright (c) 2018, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the LICENSE file, distributed with this software.
-# -----------------------------------------------------------------------------
 
 from unyt import physical_constants, unit_symbols
 from unyt.array import (  # NOQA: F401
@@ -66,14 +59,7 @@ from unyt.unit_object import Unit, define_unit  # NOQA: F401
 from unyt.unit_registry import UnitRegistry  # NOQA: F401
 from unyt.unit_systems import UnitSystem  # NOQA: F401
 
-from ._version import get_versions
-
-try:
-    from unyt.mpl_interface import matplotlib_support  # NOQA: F401
-except ImportError:
-    pass
-else:
-    matplotlib_support = matplotlib_support()
+from ._version import __version__
 
 
 # function to only import quantities into this namespace
@@ -83,18 +69,15 @@ else:
 def import_units(module, namespace):
     """Import Unit objects from a module into a namespace"""
     for key, value in module.__dict__.items():
-        if isinstance(value, (unyt_quantity, Unit)):
+        if key not in namespace and isinstance(value, (unyt_quantity, Unit)):
             namespace[key] = value
 
 
-import_units(unit_symbols, globals())
 import_units(physical_constants, globals())
+import_units(unit_symbols, globals())
 
 del import_units
 
-__version__ = get_versions()["version"]
-del get_versions
-
 
 def test():  # pragma: no cover
     """Execute the unit tests on an installed copy of unyt.
@@ -107,3 +90,9 @@ def test():  # pragma: no cover
     import pytest
 
     pytest.main([os.path.dirname(os.path.abspath(__file__))])
+
+
+# isort: off
+from unyt.mpl_interface import matplotlib_support
+
+matplotlib_support = matplotlib_support()
