@@ -1,17 +1,9 @@
-# -*- coding: utf-8 -*-
 """
 The default unit symbol lookup table.
 
 
 """
 
-# -----------------------------------------------------------------------------
-# Copyright (c) 2018, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the LICENSE file, distributed with this software.
-# -----------------------------------------------------------------------------
 
 from collections import OrderedDict, defaultdict
 
@@ -19,11 +11,16 @@ import numpy as np
 
 from unyt import dimensions
 from unyt._physical_ratios import (
+    J_per_BTU,
     J_per_eV,
+    J_per_watt_hour,
     amu_grams,
     amu_kg,
     avogadros_number,
     boltzmann_constant_J_per_K,
+    btu_per_mmbtu,
+    btu_per_quad,
+    btu_per_therm,
     elementary_charge_C,
     eps_0,
     jansky_mks,
@@ -35,6 +32,7 @@ from unyt._physical_ratios import (
     m_per_ft,
     m_per_inch,
     m_per_ly,
+    m_per_mile,
     m_per_pc,
     m_per_rearth,
     m_per_rjup,
@@ -52,6 +50,10 @@ from unyt._physical_ratios import (
     mass_uranus_kg,
     mass_venus_kg,
     metallicity_sun,
+    metallicity_sun_angr,
+    metallicity_sun_aspl,
+    metallicity_sun_feld,
+    metallicity_sun_lodd,
     mu_0,
     neper_per_bel,
     newton_mks,
@@ -72,6 +74,8 @@ from unyt._physical_ratios import (
     speed_of_light_m_per_s,
     standard_gravity_m_per_s2,
     temp_sun_kelvin,
+    uk_fl_oz_per_L,
+    us_fl_oz_per_L,
     watt_per_horsepower,
 )
 
@@ -129,12 +133,31 @@ default_unit_symbol_lut = OrderedDict(
         ),
         ("degC", (1.0, dimensions.temperature, -273.15, r"^\circ\rm{C}", True)),
         ("delta_degC", (1.0, dimensions.temperature, 0, r"\Delta^\circ\rm{C}", True)),
+        ("L", (1e-3, dimensions.volume, 0, r"\rm{L}", True)),
+        ("ha", (1.0e4, dimensions.area, 0.0, r"\rm{ha}", False)),
+        ("t", (1.0e3, dimensions.mass, 0.0, r"\rm{t}", False)),
         # Imperial and other non-metric units
         ("mil", (1e-3 * m_per_inch, dimensions.length, 0.0, r"\rm{mil}", False)),
         ("inch", (m_per_inch, dimensions.length, 0.0, r"\rm{in}", False)),
         ("ft", (m_per_ft, dimensions.length, 0.0, r"\rm{ft}", False)),
         ("yd", (0.9144, dimensions.length, 0.0, r"\rm{yd}", False)),
-        ("mile", (1609.344, dimensions.length, 0.0, r"\rm{mile}", False)),
+        ("mile", (m_per_mile, dimensions.length, 0.0, r"\rm{mile}", False)),
+        ("nmi", (m_per_mile * 1.1508, dimensions.length, 0.0, r"\rm{nmi}", False)),
+        (
+            "mph",
+            (m_per_mile / sec_per_hr, dimensions.velocity, 0.0, r"\rm{mph}", False),
+        ),
+        (
+            "kt",
+            (
+                m_per_mile * 1.1508 / sec_per_hr,
+                dimensions.velocity,
+                0.0,
+                r"\rm{kt}",
+                False,
+            ),
+        ),
+        ("acre", (4046.8564224, dimensions.area, 0.0, r"\rm{acre}", False)),
         ("furlong", (m_per_ft * 660.0, dimensions.length, 0.0, r"\rm{fur}", False)),
         (
             "degF",
@@ -184,7 +207,11 @@ default_unit_symbol_lut = OrderedDict(
         ("atm", (pascal_per_atm, dimensions.pressure, 0.0, r"\rm{atm}", False)),
         ("hp", (watt_per_horsepower, dimensions.power, 0.0, r"\rm{hp}", False)),
         ("oz", (kg_per_pound / 16.0, dimensions.mass, 0.0, r"\rm{oz}", False)),
-        ("ton", (kg_per_pound * 2000.0, dimensions.mass, 0.0, r"\rm{ton}", False)),
+        ("ton", (kg_per_pound * 2000.0, dimensions.mass, 0.0, r"\rm{US ton}", False)),
+        (
+            "ton_UK",
+            (kg_per_pound * 2240.0, dimensions.mass, 0.0, r"\rm{UK ton}", False),
+        ),
         (
             "slug",
             (
@@ -195,8 +222,38 @@ default_unit_symbol_lut = OrderedDict(
                 False,
             ),
         ),
+        ("fl_oz_US", (us_fl_oz_per_L, dimensions.volume, 0.0, r"\rm{US fl oz}", False)),
+        ("fl_oz_UK", (uk_fl_oz_per_L, dimensions.volume, 0.0, r"\rm{UK fl oz}", False)),
+        (
+            "pt_US",
+            (us_fl_oz_per_L * 16.0, dimensions.volume, 0.0, r"\rm{US pt}", False),
+        ),
+        (
+            "pt_UK",
+            (uk_fl_oz_per_L * 20.0, dimensions.volume, 0.0, r"\rm{UK pt}", False),
+        ),
+        (
+            "qt_US",
+            (us_fl_oz_per_L * 32.0, dimensions.volume, 0.0, r"\rm{US qt}", False),
+        ),
+        (
+            "qt_UK",
+            (uk_fl_oz_per_L * 40.0, dimensions.volume, 0.0, r"\rm{UK qt}", False),
+        ),
+        (
+            "gal_US",
+            (us_fl_oz_per_L * 128.0, dimensions.volume, 0.0, r"\rm{US gal}", False),
+        ),
+        (
+            "gal_UK",
+            (uk_fl_oz_per_L * 160.0, dimensions.volume, 0.0, r"\rm{UK gal}", False),
+        ),
         ("cal", (4.184, dimensions.energy, 0.0, r"\rm{cal}", True)),
-        ("BTU", (1055.0559, dimensions.energy, 0.0, r"\rm{BTU}", False)),
+        ("BTU", (J_per_BTU, dimensions.energy, 0.0, r"\rm{BTU}", False)),
+        ("MMBTU", (btu_per_mmbtu, dimensions.energy, 0.0, r"\rm{MMBTU}", False)),
+        ("therm", (btu_per_therm, dimensions.energy, 0.0, r"\rm{therm}", False)),
+        ("quad", (btu_per_quad, dimensions.energy, 0.0, r"\rm{quad}", False)),
+        ("Wh", (J_per_watt_hour, dimensions.energy, 0.0, r"\rm{Wh}", True)),
         (
             "pli",
             (
@@ -285,6 +342,11 @@ default_unit_symbol_lut = OrderedDict(
         ("min", (sec_per_min, dimensions.time, 0.0, r"\rm{min}", False)),
         ("hr", (sec_per_hr, dimensions.time, 0.0, r"\rm{hr}", False)),
         ("day", (sec_per_day, dimensions.time, 0.0, r"\rm{d}", False)),
+        ("week", (7.0 * sec_per_day, dimensions.time, 0.0, r"\rm{week}", False)),
+        (
+            "fortnight",
+            (14.0 * sec_per_day, dimensions.time, 0.0, r"\rm{fortnight}", False),
+        ),
         ("yr", (sec_per_year, dimensions.time, 0.0, r"\rm{yr}", True)),
         # Velocities
         ("c", (speed_of_light_m_per_s, dimensions.velocity, 0.0, r"\rm{c}", False)),
@@ -300,6 +362,46 @@ default_unit_symbol_lut = OrderedDict(
             "Zsun",
             (metallicity_sun, dimensions.dimensionless, 0.0, r"\rm{Z}_\odot", False),
         ),
+        (
+            "Zsun_angr",
+            (
+                metallicity_sun_angr,
+                dimensions.dimensionless,
+                0.0,
+                r"\rm{Z}_\odot",
+                False,
+            ),
+        ),
+        (
+            "Zsun_aspl",
+            (
+                metallicity_sun_aspl,
+                dimensions.dimensionless,
+                0.0,
+                r"\rm{Z}_\odot",
+                False,
+            ),
+        ),
+        (
+            "Zsun_feld",
+            (
+                metallicity_sun_feld,
+                dimensions.dimensionless,
+                0.0,
+                r"\rm{Z}_\odot",
+                False,
+            ),
+        ),
+        (
+            "Zsun_lodd",
+            (
+                metallicity_sun_lodd,
+                dimensions.dimensionless,
+                0.0,
+                r"\rm{Z}_\odot",
+                False,
+            ),
+        ),
         ("Mjup", (mass_jupiter_kg, dimensions.mass, 0.0, r"\rm{M}_{\rm{Jup}}", False)),
         ("Mearth", (mass_earth_kg, dimensions.mass, 0.0, r"\rm{M}_\oplus", False)),
         ("Rjup", (m_per_rjup, dimensions.length, 0.0, r"\rm{R}_\mathrm{Jup}", False)),
@@ -321,6 +423,10 @@ default_unit_symbol_lut = OrderedDict(
             "rpm",
             (2.0 * np.pi / 60.0, dimensions.angular_frequency, 0.0, r"\rm{RPM}", False),
         ),
+        (
+            "rev",
+            (2.0 * np.pi, dimensions.angle, 0.0, r"\rm{rev}", False),
+        ),
         # misc
         ("eV", (J_per_eV, dimensions.energy, 0.0, r"\rm{eV}", True)),
         ("amu", (amu_kg, dimensions.mass, 0.0, r"\rm{amu}", False)),
@@ -340,7 +446,7 @@ default_unit_symbol_lut = OrderedDict(
         ("nt", (1.0, dimensions.luminance, 0.0, r"\rm{nt}", False)),
         # Planck units
         ("m_pl", (planck_mass_kg, dimensions.mass, 0.0, r"m_{\rm{P}}", False)),
-        ("l_pl", (planck_length_m, dimensions.length, 0.0, r"\ell_\rm{P}", False)),
+        ("l_pl", (planck_length_m, dimensions.length, 0.0, r"\ell_{\rm{P}}", False)),
         ("t_pl", (planck_time_s, dimensions.time, 0.0, r"t_{\rm{P}}", False)),
         (
             "T_pl",
@@ -418,7 +524,6 @@ default_base_units = {
 physical_constants = OrderedDict(
     [
         ("me", (mass_electron_kg, "kg", ["mass_electron", "electron_mass"])),
-        ("amu", (amu_kg, "kg", ["atomic_mass_unit"])),
         ("Na", (avogadros_number, "mol**-1", ["Avogadros_number", "avogadros_number"])),
         ("mp", (mass_proton_kg, "kg", ["proton_mass", "mass_proton"])),
         ("mh", (mass_hydrogen_kg, "kg", ["hydrogen_mass", "mass_hydrogen"])),
@@ -530,6 +635,15 @@ default_unit_name_alternatives = OrderedDict(
         ("lm", ("lumen",)),
         ("lx", ("lux",)),
         ("degC", ("degree_celsius", "degree_Celsius", "celcius", "celsius", "°C")),
+        ("L", ("liter", "litre", "l")),
+        ("ha", ("hectare",)),
+        (
+            "t",
+            (
+                "tonne",
+                "metric_ton",
+            ),
+        ),
         # Imperial and other non-metric units
         ("mil", ("thou", "thousandth")),
         ("inch", ("in",)),
@@ -541,11 +655,39 @@ default_unit_name_alternatives = OrderedDict(
         ("lbf", ("pound_force",)),
         ("kip", ("kilopound", "kipf")),
         ("lb", ("pound", "pound_mass", "lbm")),
+        ("ton", ("ton_US", "ton_US_short", "short_ton")),
+        ("ton_UK", ("ton_US_long", "long_ton")),
         ("atm", ("atmosphere",)),
         ("hp", ("horsepower",)),
         ("oz", ("ounce",)),
+        ("nmi", ("nautical_mile",)),
+        ("fl_oz_US", ("fluid_ounce_US",)),
+        ("fl_oz_UK", ("fluid_ounce_UK",)),
+        ("pt_US", ("pint_US",)),
+        ("pt_UK", ("pint_UK",)),
+        ("qt_US", ("quart_US",)),
+        ("qt_UK", ("quart_UK",)),
+        ("gal_US", ("gallon_US",)),
+        ("gal_UK", ("gallon_UK",)),
+        ("kt", ("knot",)),
         ("cal", ("calorie",)),
-        ("BTU", ("british_thermal_unit",)),
+        (
+            "BTU",
+            (
+                "british_thermal_unit",
+                "btu",
+            ),
+        ),
+        ("MMBTU", ("mmbtu",)),
+        ("therm", ("therms",)),
+        ("quad", ("quads",)),
+        (
+            "Wh",
+            (
+                "watt_hour",
+                "watt_hours",
+            ),
+        ),
         ("pli", ("pounds_per_inch",)),
         ("plf", ("pounds_per_ft",)),
         ("psi", ("pounds_per_square_inch",)),
@@ -598,6 +740,7 @@ default_unit_name_alternatives = OrderedDict(
         ("sr", ("steradian",)),
         ("lat", ("latitude", "degree_latitude")),
         ("lon", ("longitude", "degree_longitude")),
+        ("rev", ("revolution",)),
         # misc
         ("eV", ("electronvolt",)),
         ("amu", ("atomic_mass_unit",)),
@@ -634,7 +777,7 @@ def generate_name_alternatives():
         else:
             if okey[0] not in ["u", "μ"]:
                 raise RuntimeError(  # pragma: no cover
-                    "Duplicate unit name found: {}, {}".format(key, okey)
+                    f"Duplicate unit name found: {key}, {okey}"
                 )
 
     for key, entry in default_unit_symbol_lut.items():
@@ -650,7 +793,7 @@ def generate_name_alternatives():
                     used_prefix = prefix
                 append_name(names[prefix + key], used_prefix + key, prefix + key)
         elif len(key) > 3 and key.title() != key:
-            if all([len(k) > 3 for k in key.split("_")]):
+            if all(len(k) > 3 for k in key.split("_")):
                 append_name(names[key], key, key.title())
         if key in default_unit_name_alternatives:
             alternatives = default_unit_name_alternatives[key]
