@@ -5,16 +5,11 @@ unyt_array class.
 
 """
 
-# -----------------------------------------------------------------------------
-# Copyright (c) 2018, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the LICENSE file, distributed with this software.
-# -----------------------------------------------------------------------------
 
 import copy
 import re
+import sys
+import warnings
 from functools import lru_cache
 from numbers import Number as numeric_type
 
@@ -33,6 +28,7 @@ from numpy import (
     bitwise_or,
     bitwise_xor,
     ceil,
+    clip,
     conj,
     copysign,
     cos,
@@ -106,17 +102,9 @@ from numpy import (
     true_divide,
     trunc,
 )
-from numpy.core.umath import _ones_like
-
-try:
-    from numpy.core.umath import clip
-except ImportError:
-    clip = None
-import warnings
-
 from sympy import Rational
 
-from unyt._on_demand_imports import _astropy, _pint
+from unyt._on_demand_imports import _astropy, _dask, _pint
 from unyt._pint_conversions import convert_pint_units
 from unyt._unit_lookup_table import default_unit_symbol_lut
 from unyt.dimensions import angle, temperature
@@ -138,6 +126,9 @@ from unyt.unit_registry import (
     _sanitize_unit_system,
     default_unit_registry,
 )
+from unyt.unit_symbols import delta_degC, delta_degF
+
+from ._deprecation import warn_deprecated
 
 NULL_UNIT = Unit()
 POWER_MAPPING = {multiply: lambda x: x, divide: lambda x: 2 - x}
@@ -194,25 +185,51 @@ def _multiply_units(unit1, unit2):
     return ret.as_coeff_unit()
 
 
-TEMPERATURE_WARNING = """
-    Ambiguous operation with heterogeneous temperature units.
-    In the future, such operations will generate UnitOperationError.
-    Use delta_degC or delta_degF to avoid the ambiguity.
-"""
-
-
 @lru_cache(maxsize=128, typed=False)
 def _preserve_units(unit1, unit2=None):
     if unit2 is None or unit1.dimensions is not temperature:
         return 1, unit1
     if unit1.base_offset == 0.0 and unit2.base_offset != 0.0:
-        if str(unit1.expr) in ["K", "R"]:
-            warnings.warn(TEMPERATURE_WARNING, FutureWarning, stacklevel=3)
-            return 1, unit1
         return 1, unit2
     return 1, unit1
 
 
+@lru_cache(maxsize=128, typed=False)
+def _difference_units(unit1, unit2=None):
+    if unit1.dimensions is not temperature:
+        return _preserve_units(unit1, unit2)
+
+    s1 = repr(unit1)
+    if unit2 is not None and unit2 != unit1:
+        s2 = repr(unit2)
+        if s1 in s2 and s2.startswith("delta_"):
+            return 1, unit1
+        elif s2 in s1 and s1.startswith("delta_"):
+            return 1, unit2
+        else:
+            raise InvalidUnitOperation(
+                "Quantities with units of Fahrenheit or Celsius "
+                "cannot be multiplied, divided, subtracted or "
+                "added with data that has different units."
+            )
+
+    if unit1.base_offset == 0.0:
+        return 1, unit1
+
+    if s1 == "degF":
+        return 1, delta_degF
+    elif s1 == "degC":
+        return 1, delta_degC
+    else:
+        # This is supposed to be unreachable
+        raise RuntimeError(
+            "Could not determine difference temperature units "
+            f"in operation ({unit1} - {unit2}).\n"
+            "If you see this error please file an issue at "
+            "https://github.com/yt-project/unyt/issues/new"
+        )
+
+
 @lru_cache(maxsize=128, typed=False)
 def _power_unit(unit, power):
     return 1, unit**power
@@ -297,6 +314,18 @@ def _sanitize_units_convert(possible_units, registry):
     return unit
 
 
+def _apply_power_mapping(ufunc, in_unit, in_size, in_shape, input_kwarg_dict):
+    # a reduction of a multiply or divide corresponds to
+    # a repeated product which we implement as an exponent
+    mul = 1
+    power_map = POWER_MAPPING[ufunc]
+    if input_kwarg_dict.get("axis", None) is not None:
+        unit = in_unit ** (power_map(in_shape[input_kwarg_dict["axis"]]))
+    else:
+        unit = in_unit ** (power_map(in_size))
+    return mul, unit
+
+
 unary_operators = (
     negative,
     absolute,
@@ -402,11 +431,11 @@ class unyt_array(np.ndarray):
 
     input_array : iterable
         A tuple, list, or array to attach units to
-    input_units : String unit name, unit symbol object, or astropy unit
+    units : String unit name, unit symbol object, or astropy unit
         The units of the array. Powers must be specified using python
         syntax (cm**3, not cm^3).
     registry : :class:`unyt.unit_registry.UnitRegistry`
-        The registry to create units from. If input_units is already associated
+        The registry to create units from. If units is already associated
         with a unit registry and this is specified, this will be used instead
         of the registry associated with the unit object.
     dtype : numpy dtype or dtype name
@@ -416,7 +445,7 @@ class unyt_array(np.ndarray):
         If True, all input validation is skipped. Using this option may produce
         corrupted, invalid units or array data, but can lead to significant
         speedups in the input validation logic adds significant overhead. If
-        set, input_units *must* be a valid unit object. Defaults to False.
+        set, units *must* be a valid unit object. Defaults to False.
     name : string
         The name of the array. Defaults to None. This attribute does not propagate
         through mathematical operations, but is preserved under indexing
@@ -451,7 +480,7 @@ class unyt_array(np.ndarray):
 
     _ufunc_registry = {
         add: _preserve_units,
-        subtract: _preserve_units,
+        subtract: _difference_units,
         multiply: _multiply_units,
         divide: _divide_units,
         logaddexp: _return_without_unit,
@@ -533,7 +562,6 @@ class unyt_array(np.ndarray):
         divmod_: _passthrough_unit,
         isnat: _return_without_unit,
         heaviside: _preserve_units,
-        _ones_like: _preserve_units,
         matmul: _multiply_units,
         clip: _passthrough_unit,
     }
@@ -546,19 +574,11 @@ class unyt_array(np.ndarray):
         units=None,
         registry=None,
         dtype=None,
+        *,
         bypass_validation=False,
-        input_units=None,
         name=None,
     ):
-        # deprecate input_units in favor of units
-        if input_units is not None:
-            warnings.warn(
-                "input_units has been deprecated, please use units instead",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-        if units is not None:
-            input_units = units
+        input_units = units
         if bypass_validation is True:
             if dtype is None:
                 dtype = input_array.dtype
@@ -614,18 +634,13 @@ class unyt_array(np.ndarray):
         return obj
 
     def __repr__(self):
-        rep = super(unyt_array, self).__repr__()
-        units_repr = self.units.__repr__()
-        if "=" in rep:
-            return rep[:-1] + ", units='" + units_repr + "')"
-        else:
-            return rep[:-1] + ", '" + units_repr + "')"
+        return np.array_repr(self)
 
     def __str__(self):
         return str(self.view(np.ndarray)) + " " + str(self.units)
 
     def __format__(self, format_spec):
-        return "{} {}".format(self.d.__format__(format_spec), self.units)
+        return f"{self.d.__format__(format_spec)} {self.units}"
 
     #
     # Start unit conversion methods
@@ -698,8 +713,7 @@ class unyt_array(np.ndarray):
                 large = LARGE_INPUT.get(dsize, 0)
                 if large and np.any(np.abs(values) > large):
                     warnings.warn(
-                        "Overflow encountered while converting to units '%s'"
-                        % new_units,
+                        f"Overflow encountered while converting to units '{new_units}'",
                         RuntimeWarning,
                         stacklevel=2,
                     )
@@ -879,8 +893,7 @@ class unyt_array(np.ndarray):
                 large = LARGE_INPUT.get(dsize, 0)
                 if large and np.any(np.abs(self.d) > large):
                     warnings.warn(
-                        "Overflow encountered while converting to units '%s'"
-                        % new_units,
+                        f"Overflow encountered while converting to units '{new_units}'",
                         RuntimeWarning,
                         stacklevel=2,
                     )
@@ -1275,7 +1288,7 @@ class unyt_array(np.ndarray):
             # hour as "h"
             if unit_str == "h":
                 unit_str = "hr"
-            ap_units.append("%s**(%s)" % (unit_str, Rational(exponent)))
+            ap_units.append(f"{unit_str}**({Rational(exponent)})")
         ap_units = "*".join(ap_units)
         if isinstance(_arr.value, np.ndarray) and _arr.shape != ():
             return unyt_array(_arr.value, ap_units, registry=unit_registry)
@@ -1293,7 +1306,11 @@ class unyt_array(np.ndarray):
         >>> data.to_astropy()
         <Quantity [3., 4., 5.] g / cm3>
         """
-        return self.value * _astropy.units.Unit(str(self.units), **kwargs)
+        if self.units.is_dimensionless:
+            s_units = ""
+        else:
+            s_units = str(self.units)
+        return self.value * _astropy.units.Unit(s_units, **kwargs)
 
     @classmethod
     def from_pint(cls, arr, unit_registry=None):
@@ -1324,7 +1341,7 @@ class unyt_array(np.ndarray):
         p_units = []
         for base, exponent in arr._units.items():
             bs = convert_pint_units(base)
-            p_units.append("%s**(%s)" % (bs, Rational(exponent)))
+            p_units.append(f"{bs}**({Rational(exponent)})")
         p_units = "*".join(p_units)
         if isinstance(arr.magnitude, np.ndarray):
             return unyt_array(arr.magnitude, p_units, registry=unit_registry)
@@ -1362,7 +1379,7 @@ class unyt_array(np.ndarray):
             # "yr" as "year"
             if str(unit).endswith("yr") and len(str(unit)) in [2, 3]:
                 unit = str(unit).replace("yr", "year")
-            units.append("%s**(%s)" % (unit, Rational(pow)))
+            units.append(f"{unit}**({Rational(pow)})")
         units = "*".join(units)
         return unit_registry.Quantity(self.value, units)
 
@@ -1719,13 +1736,10 @@ class unyt_array(np.ndarray):
         return np.ones_like(self)
 
     def __getitem__(self, item):
-        ret = super(unyt_array, self).__getitem__(item)
+        ret = super().__getitem__(item)
         if getattr(ret, "shape", None) == ():
             ret = unyt_quantity(ret, bypass_validation=True, name=self.name)
-        try:
             ret.units = self.units
-        except AttributeError:
-            pass
         return ret
 
     def __setitem__(self, item, value):
@@ -1736,12 +1750,9 @@ class unyt_array(np.ndarray):
 
     def __pow__(self, p, mod=None, /):
         """
-        Power function, over-rides the ufunc as
-        ``numpy`` passes the ``_ones_like`` ufunc for
-        powers of zero (in some cases), which leads to an
-        incorrect unit calculation.
+        Power function
         """
-
+        # see https://github.com/yt-project/unyt/issues/203
         if p == 0.0:
             ret = self.ua
             ret.units = Unit("dimensionless")
@@ -1801,14 +1812,7 @@ class unyt_array(np.ndarray):
             # evaluate the ufunc
             out_arr = func(np.asarray(inp), out=out_func, **kwargs)
             if ufunc in (multiply, divide) and method == "reduce":
-                # a reduction of a multiply or divide corresponds to
-                # a repeated product which we implement as an exponent
-                mul = 1
-                power_map = POWER_MAPPING[ufunc]
-                if "axis" in kwargs and kwargs["axis"] is not None:
-                    unit = u ** (power_map(inp.shape[kwargs["axis"]]))
-                else:
-                    unit = u ** (power_map(inp.size))
+                mul, unit = _apply_power_mapping(ufunc, u, inp.size, inp.shape, kwargs)
             else:
                 # get unit of result
                 mul, unit = self._ufunc_registry[ufunc](u)
@@ -1819,6 +1823,14 @@ class unyt_array(np.ndarray):
             # binary ufuncs
             i0 = inputs[0]
             i1 = inputs[1]
+
+            if "dask" in sys.modules and isinstance(i1, _dask.array.core.Array):
+                # need to short circuit all this to handle binary operations
+                # like unyt_quantity(2,'m') / unyt_dask_array_instance
+                # only need to check the second argument as if the first arg
+                # is a unyt_dask_array, it won't end up here.
+                return i1.__array_ufunc__(ufunc, method, *inputs, **kwargs)
+
             # coerce inputs to be ndarrays if they aren't already
             inp0 = _coerce_iterable_units(i0)
             inp1 = _coerce_iterable_units(i1)
@@ -1843,7 +1855,23 @@ class unyt_array(np.ndarray):
                 else:
                     u1 = 1.0
             unit_operator = self._ufunc_registry[ufunc]
-            if unit_operator in (_preserve_units, _comparison_unit, _arctan2_unit):
+
+            if (
+                unit_operator is _preserve_units
+                and u0.dimensions is temperature
+                and u1 is not None
+                and u1.base_offset != 0.0
+                and u0.base_offset == 0.0
+                and str(u0.expr) in ["K", "R"]
+            ):
+                raise UnitOperationError(ufunc, u0, u1)
+
+            if unit_operator in (
+                _preserve_units,
+                _comparison_unit,
+                _arctan2_unit,
+                _difference_units,
+            ):
                 # check "is" equality first for speed
                 if u0 is not u1 and u0 != u1:
                     # we allow adding, multiplying, comparisons with
@@ -1893,10 +1921,14 @@ class unyt_array(np.ndarray):
                     conv, offset = u1.get_conversion_factor(u0, inp1.dtype)
                     new_dtype = np.dtype("f" + str(inp1.dtype.itemsize))
                     conv = new_dtype.type(conv)
-                    if offset is not None:
+                    if (
+                        offset is not None
+                        and u1.base_offset != 0.0
+                        and not repr(u0).startswith("delta_")
+                    ):
                         raise InvalidUnitOperation(
                             "Quantities with units of Fahrenheit or Celsius "
-                            "cannot by multiplied, divided, subtracted or "
+                            "cannot be multiplied, divided, subtracted or "
                             "added with data that has different units."
                         )
                     inp1 = np.asarray(inp1, dtype=new_dtype) * conv
@@ -1922,7 +1954,7 @@ class unyt_array(np.ndarray):
                 ):
                     raise InvalidUnitOperation(
                         "Quantities with units of Fahrenheit or Celsius "
-                        "cannot by multiplied, divide, subtracted or added."
+                        "cannot be multiplied, divided, subtracted or added."
                     )
         else:
             if ufunc is clip:
@@ -1948,7 +1980,7 @@ class unyt_array(np.ndarray):
         if unit is None:
             out_arr = np.array(out_arr, copy=False)
         elif ufunc in (modf, divmod_):
-            out_arr = tuple((ret_class(o, unit) for o in out_arr))
+            out_arr = tuple(ret_class(o, unit) for o in out_arr)
         elif out_arr.shape == ():
             out_arr = unyt_quantity(np.asarray(out_arr), unit)
         elif out_arr.size == 1:
@@ -1976,14 +2008,31 @@ class unyt_array(np.ndarray):
                 for o, oa in zip(out, out_arr):
                     if o is None:
                         continue
-                    try:
-                        o.units = oa.units
-                    except AttributeError:
-                        o.units = Unit("", registry=self.units.registry)
+                    o.units = oa.units
         if mul == 1:
             return out_arr
         return mul * out_arr
 
+    def __array_function__(self, func, types, args, kwargs):
+        # Follow NEP 18 guidelines
+        # https://numpy.org/neps/nep-0018-array-function-protocol.html
+        from unyt._array_functions import _HANDLED_FUNCTIONS, _UNSUPPORTED_FUNCTIONS
+
+        if func in _UNSUPPORTED_FUNCTIONS:
+            # following NEP 18, return NotImplemented as a sentinel value
+            # which will lead to raising a TypeError, while
+            # leaving other arguments a chance to take the lead
+            return NotImplemented
+
+        if func not in _HANDLED_FUNCTIONS:
+            # default to numpy's private implementation
+            return func._implementation(*args, **kwargs)
+        # Note: this allows subclasses that don't override
+        # __array_function__ to handle unyt_array objects
+        if not all(issubclass(t, unyt_array) or t is np.ndarray for t in types):
+            return NotImplemented
+        return _HANDLED_FUNCTIONS[func](*args, **kwargs)
+
     def copy(self, order="C"):
         """
         Return a copy of the array.
@@ -2033,7 +2082,7 @@ class unyt_array(np.ndarray):
         """Posify the data."""
         # this needs to be defined for all numpy versions, see
         # numpy issue #9081
-        return type(self)(super(unyt_array, self).__pos__(), self.units)
+        return type(self)(super().__pos__(), self.units)
 
     def dot(self, b, out=None):
         """dot product of two arrays.
@@ -2075,7 +2124,7 @@ class unyt_array(np.ndarray):
         returned tuple, itself a tuple used to restore the state of the
         ndarray. This is always defined for numpy arrays.
         """
-        np_ret = super(unyt_array, self).__reduce__()
+        np_ret = super().__reduce__()
         obj_state = np_ret[2]
         unit_state = (((str(self.units), self.units.registry.lut),) + obj_state[:],)
         new_ret = np_ret[:2] + unit_state + np_ret[3:]
@@ -2087,7 +2136,7 @@ class unyt_array(np.ndarray):
         This is called inside pickle.read() and restores the unit data from the
         metadata extracted in __reduce__ and then serialized by pickle.
         """
-        super(unyt_array, self).__setstate__(state[1:])
+        super().__setstate__(state[1:])
         unit, lut = state[0]
         lut = _correct_old_unit_registry(lut)
         registry = UnitRegistry(lut=lut, add_default_symbols=False)
@@ -2098,7 +2147,7 @@ class unyt_array(np.ndarray):
 
         This is necessary for stdlib deepcopy of arrays and quantities.
         """
-        ret = super(unyt_array, self).__deepcopy__(memodict)
+        ret = super().__deepcopy__(memodict)
         try:
             return type(self)(ret, copy.deepcopy(self.units), name=self.name)
         except TypeError:
@@ -2115,16 +2164,20 @@ class unyt_quantity(unyt_array):
 
     input_scalar : an integer or floating point scalar
         The scalar to attach units to
-    input_units : String unit specification, unit symbol object, or astropy
-                  units
+    units : String unit specification, unit symbol object, or astropy units
         The units of the quantity. Powers must be specified using python syntax
         (cm**3, not cm^3).
     registry : A UnitRegistry object
-        The registry to create units from. If input_units is already associated
+        The registry to create units from. If units is already associated
         with a unit registry and this is specified, this will be used instead
         of the registry associated with the unit object.
     dtype : data-type
         The dtype of the array data.
+    bypass_validation : boolean
+        If True, all input validation is skipped. Using this option may produce
+        corrupted, invalid units or array data, but can lead to significant
+        speedups in the input validation logic adds significant overhead. If
+        set, units *must* be a valid unit object. Defaults to False.
     name : string
         The name of the scalar. Defaults to None. This attribute does not propagate
         through mathematical operations, but is preserved under indexing
@@ -2161,18 +2214,11 @@ class unyt_quantity(unyt_array):
         units=None,
         registry=None,
         dtype=None,
+        *,
         bypass_validation=False,
-        input_units=None,
         name=None,
     ):
-        if input_units is not None:
-            warnings.warn(
-                "input_units has been deprecated, please use units instead",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-        if units is not None:
-            input_units = units
+        input_units = units
         if not (
             bypass_validation
             or isinstance(input_scalar, (numeric_type, np.number, np.ndarray))
@@ -2236,11 +2282,16 @@ def uconcatenate(arrs, axis=0):
     >>> from unyt import cm
     >>> A = [1, 2, 3]*cm
     >>> B = [2, 3, 4]*cm
-    >>> uconcatenate((A, B))
+    >>> uconcatenate((A, B)) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     unyt_array([1, 2, 3, 2, 3, 4], 'cm')
 
     """
-    v = np.concatenate(arrs, axis=axis)
+    warn_deprecated(
+        "unyt.uconcatenate", replacement="use numpy.concatenate", since_version="3.0"
+    )
+    v = np.concatenate._implementation(arrs, axis=axis)
     v = _validate_numpy_wrapper_units(v, arrs)
     return v
 
@@ -2252,7 +2303,16 @@ def ucross(arr1, arr2, registry=None, axisa=-1, axisb=-1, axisc=-1, axis=None):
     See the documentation of numpy.cross for full
     details.
     """
-    v = np.cross(arr1, arr2, axisa=axisa, axisb=axisb, axisc=axisc, axis=axis)
+    warn_deprecated(
+        "unyt.ucross",
+        replacement=(
+            "use numpy.cross (note that the *registry* argument will not be ported)"
+        ),
+        since_version="3.0",
+    )
+    v = np.cross._implementation(
+        arr1, arr2, axisa=axisa, axisb=axisb, axisc=axisc, axis=axis
+    )
     units = arr1.units * arr2.units
     arr = unyt_array(v, units, registry=registry)
     return arr
@@ -2270,13 +2330,16 @@ def uintersect1d(arr1, arr2, assume_unique=False):
     >>> from unyt import cm
     >>> A = [1, 2, 3]*cm
     >>> B = [2, 3, 4]*cm
-    >>> uintersect1d(A, B)
+    >>> uintersect1d(A, B) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     unyt_array([2, 3], 'cm')
 
     """
-    v = np.intersect1d(arr1, arr2, assume_unique=assume_unique)
-    v = _validate_numpy_wrapper_units(v, [arr1, arr2])
-    return v
+    warn_deprecated(
+        "unyt.uintersect1d", replacement="use numpy.intersect1d", since_version="3.0"
+    )
+    return np.intersect1d(arr1, arr2, assume_unique=assume_unique)
 
 
 def uunion1d(arr1, arr2):
@@ -2291,13 +2354,16 @@ def uunion1d(arr1, arr2):
     >>> from unyt import cm
     >>> A = [1, 2, 3]*cm
     >>> B = [2, 3, 4]*cm
-    >>> uunion1d(A, B)
+    >>> uunion1d(A, B) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     unyt_array([1, 2, 3, 4], 'cm')
 
     """
-    v = np.union1d(arr1, arr2)
-    v = _validate_numpy_wrapper_units(v, [arr1, arr2])
-    return v
+    warn_deprecated(
+        "unyt.uunion1d", replacement="use numpy.union1d", since_version="3.0"
+    )
+    return np.union1d(arr1, arr2)
 
 
 def unorm(data, ord=None, axis=None, keepdims=False):
@@ -2311,10 +2377,15 @@ def unorm(data, ord=None, axis=None, keepdims=False):
     --------
     >>> from unyt import km
     >>> data = [1, 2, 3]*km
-    >>> print(unorm(data))
+    >>> print(unorm(data)) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     3.7416573867739413 km
     """
-    norm = np.linalg.norm(data, ord=ord, axis=axis, keepdims=keepdims)
+    warn_deprecated(
+        "unyt.unorm", replacement="use numpy.linalg.norm", since_version="3.0"
+    )
+    norm = np.linalg.norm._implementation(data, ord=ord, axis=axis, keepdims=keepdims)
     if norm.shape == ():
         return unyt_quantity(norm, data.units)
     return unyt_array(norm, data.units)
@@ -2330,11 +2401,14 @@ def udot(op1, op2):
     >>> from unyt import km, s
     >>> a = np.eye(2)*km
     >>> b = (np.ones((2, 2)) * 2)*s
-    >>> print(udot(a, b))
+    >>> print(udot(a, b)) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     [[2. 2.]
      [2. 2.]] km*s
     """
-    dot = np.dot(op1.d, op2.d)
+    warn_deprecated("unyt.udot", replacement="use numpy.dot", since_version="3.0")
+    dot = np.dot._implementation(op1.d, op2.d)
     units = op1.units * op2.units
     if dot.shape == ():
         return unyt_quantity(dot, units)
@@ -2351,11 +2425,14 @@ def uvstack(arrs):
     >>> from unyt import km
     >>> a = [1, 2, 3]*km
     >>> b = [2, 3, 4]*km
-    >>> print(uvstack([a, b]))
+    >>> print(uvstack([a, b])) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     [[1 2 3]
      [2 3 4]] km
     """
-    v = np.vstack(arrs)
+    warn_deprecated("unyt.uvstack", replacement="use numpy.vstack", since_version="3.0")
+    v = np.vstack._implementation(arrs)
     v = _validate_numpy_wrapper_units(v, arrs)
     return v
 
@@ -2370,16 +2447,21 @@ def uhstack(arrs):
     >>> from unyt import km
     >>> a = [1, 2, 3]*km
     >>> b = [2, 3, 4]*km
-    >>> print(uhstack([a, b]))
+    >>> print(uhstack([a, b])) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     [1 2 3 2 3 4] km
     >>> a = [[1],[2],[3]]*km
     >>> b = [[2],[3],[4]]*km
-    >>> print(uhstack([a, b]))
+    >>> print(uhstack([a, b])) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     [[1 2]
      [2 3]
      [3 4]] km
     """
-    v = np.hstack(arrs)
+    warn_deprecated("unyt.uhstack", replacement="use numpy.hstack", since_version="3.0")
+    v = np.hstack._implementation(arrs)
     v = _validate_numpy_wrapper_units(v, arrs)
     return v
 
@@ -2399,11 +2481,14 @@ def ustack(arrs, axis=0):
     >>> from unyt import km
     >>> a = [1, 2, 3]*km
     >>> b = [2, 3, 4]*km
-    >>> print(ustack([a, b]))
+    >>> print(ustack([a, b])) # doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    DeprecationWarning: ...
     [[1 2 3]
      [2 3 4]] km
     """
-    v = np.stack(arrs, axis=axis)
+    warn_deprecated("unyt.ustack", replacement="use numpy.stack", since_version="3.0")
+    v = np.stack._implementation(arrs, axis=axis)
     v = _validate_numpy_wrapper_units(v, arrs)
     return v
 
@@ -2430,7 +2515,7 @@ def _get_binary_op_return_class(cls1, cls2):
     else:
         raise RuntimeError(
             "Undefined operation for a unyt_array subclass. "
-            "Received operand types (%s) and (%s)" % (cls1, cls2)
+            f"Received operand types ({cls1}) and ({cls2})"
         )
 
 
@@ -2461,7 +2546,7 @@ def loadtxt(fname, dtype="float", delimiter="\t", usecols=None, comments="#"):
     >>> temp, velx = loadtxt(
     ...    "sphere.dat", usecols=(1,2), delimiter="\t")  # doctest: +SKIP
     """
-    f = open(fname, "r")
+    f = open(fname)
     next_one = False
     units = []
     num_cols = -1
@@ -2499,7 +2584,7 @@ def loadtxt(fname, dtype="float", delimiter="\t", usecols=None, comments="#"):
         arrays = [arrays]
     if usecols is not None:
         units = [units[col] for col in usecols]
-    ret = tuple([unyt_array(arr, unit) for arr, unit in zip(arrays, units)])
+    ret = tuple(unyt_array(arr, unit) for arr, unit in zip(arrays, units))
     if len(ret) == 1:
         return ret[0]
     return ret
@@ -2616,7 +2701,7 @@ def allclose_units(actual, desired, rtol=1e-7, atol=0, **kwargs):
 
     rt = unyt_array(rtol)
     if not rt.units.is_dimensionless:
-        raise RuntimeError("Units of rtol (%s) are not " "dimensionless" % rt.units)
+        raise RuntimeError(f"Units of rtol ({rt.units}) are not dimensionless")
 
     if not isinstance(atol, unyt_array):
         at = unyt_quantity(atol, des.units)
