@@ -3,15 +3,6 @@ Equivalencies between different kinds of units
 
 """
 
-# -----------------------------------------------------------------------------
-# Copyright (c) 2018, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the LICENSE file, distributed with this software.
-# -----------------------------------------------------------------------------
-
-from __future__ import division
 
 from collections import OrderedDict
 
@@ -42,7 +33,7 @@ class _RegisteredEquivalence(type):
             equivalence_registry[cls.type_name] = cls
 
 
-class Equivalence(object, metaclass=_RegisteredEquivalence):
+class Equivalence(metaclass=_RegisteredEquivalence):
     def __init__(self, in_place=False):
         self.in_place = in_place
 
@@ -338,7 +329,7 @@ class LorentzEquivalence(Equivalence):
 
       \\gamma = \\frac{1}{\\sqrt{1 - v^2/c^2}}
 
-    ans similarly
+    and similarly
 
     .. math::
 
