@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Test symbolic unit handling.
 
@@ -7,14 +6,6 @@ Test symbolic unit handling.
 
 """
 
-# -----------------------------------------------------------------------------
-# Copyright (c) 2018, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the LICENSE file, distributed with this software.
-# -----------------------------------------------------------------------------
-
 
 import operator
 import pickle
@@ -75,10 +66,10 @@ def test_no_conflicting_symbols():
         else:
             keys = [symbol]
         for prefix in keys:
-            new_symbol = "%s%s" % (prefix, symbol)
+            new_symbol = f"{prefix}{symbol}"
 
             # test if we have seen this symbol
-            assert new_symbol not in full_set, "Duplicate symbol: %s" % new_symbol
+            assert new_symbol not in full_set, f"Duplicate symbol: {new_symbol}"
 
             full_set.add(new_symbol)
 
@@ -513,15 +504,25 @@ def test_base_equivalent():
     assert u.get_base_equivalent(unit_system="mks") == Unit("A")
 
 
-def test_temperature_offsets():
-    u1 = Unit("degC")
-    u2 = Unit("degF")
-
+@pytest.mark.parametrize(
+    ("u1", "u2"),
+    [
+        (Unit("degC"), Unit("degF")),
+        (Unit("degC"), Unit("K")),
+        (Unit("degF"), Unit("K")),
+    ],
+)
+def test_temperature_offsets(u1, u2):
     with pytest.raises(InvalidUnitOperation):
         operator.mul(u1, u2)
     with pytest.raises(InvalidUnitOperation):
         operator.truediv(u1, u2)
 
+    assert u1 != u2
+    assert u2 != u1
+    assert not u1 == u2
+    assert not u2 == u1
+
 
 def test_latex_repr():
     registry = UnitRegistry()
@@ -574,7 +575,7 @@ def test_latitude_longitude():
     assert_equal((lon * 180.0).in_units("deg"), deg * 360)
 
 
-def test_creation_from_ytarray():
+def test_creation_from_unyt_array():
     from unyt import electrostatic_unit, elementary_charge_cgs
 
     u1 = Unit(electrostatic_unit)
@@ -893,7 +894,6 @@ def test_delta_degF():
 
 
 def test_mixed_registry_operations():
-
     reg = UnitRegistry(unit_system="cgs")
     reg.add("fake_length", 0.001, length)
     a = unyt_quantity(1, units="fake_length", registry=reg)
