@@ -5,15 +5,6 @@ Test ndarray subclass that handles symbolic units.
 
 
 """
-
-# ----------------------------------------------------------------------------
-# Copyright (c) 2013, yt Development Team.
-#
-# Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the file COPYING.txt, distributed with this software.
-# ----------------------------------------------------------------------------
-
 import copy
 import itertools
 import math
@@ -23,13 +14,11 @@ import pickle
 import re
 import shutil
 import tempfile
-import warnings
 from importlib.metadata import version
 from pathlib import Path
 
 import numpy as np
 import pytest
-import sympy
 from numpy import array
 from numpy.testing import (
     assert_almost_equal,
@@ -40,7 +29,7 @@ from numpy.testing import (
 from packaging.version import Version
 
 from unyt import K, R, Unit, degC, degF, delta_degC, delta_degF, dimensions
-from unyt._on_demand_imports import NotAModule, _astropy, _h5py, _pint
+from unyt._on_demand_imports import _astropy, _h5py, _pint
 from unyt._physical_ratios import metallicity_sun, speed_of_light_cm_per_s
 from unyt.array import (
     binary_operators,
@@ -67,7 +56,11 @@ from unyt.exceptions import (
     UnitParseError,
     UnitsNotReducible,
 )
-from unyt.testing import _process_warning, assert_allclose_units
+from unyt.testing import (
+    _process_warning,
+    assert_allclose_units,
+    assert_array_equal_units,
+)
 from unyt.unit_registry import UnitRegistry
 from unyt.unit_symbols import cm, degree, g, m
 
@@ -529,8 +522,10 @@ def test_power():
         assert isinstance(err.unit1, Unit)
         assert isinstance(err.unit2, Unit)
 
-    assert_equal(cm_quant, unyt_quantity(1.0, "dimensionless"))
-    assert_equal(cm_quant, unyt_quantity(1.0, "dimensionless"))
+    # when the power is 0.0 numpy short-circuits via ones_like so we
+    # need to test the special handling for that case
+    assert_array_equal_units(cm_quant**0, unyt_quantity(1.0, "dimensionless"))
+    assert_array_equal_units(cm_arr**0, unyt_quantity(1.0, "dimensionless"))
 
 
 def test_comparisons():
@@ -834,7 +829,7 @@ def test_iteration():
         assert_equal(ib.units, b.units)
 
 
-def test_ytarray_pickle():
+def test_unyt_array_pickle():
     test_data = [unyt_quantity(12.0, "cm"), unyt_array([1, 2, 3], "km")]
 
     for data in test_data:
@@ -852,11 +847,17 @@ def test_ytarray_pickle():
         assert_equal(float(data.units.base_value), float(loaded_data.units.base_value))
 
 
+SYMPY_VERSION = Version(version("sympy"))
+
+
 @pytest.mark.xfail(
-    condition=(
-        (Version(sympy.__version__) == Version("1.9"))
-        or (Version(sympy.__version__) == Version("1.10"))
-    ),
+    condition=(SYMPY_VERSION == Version("1.12")),
+    reason="regression in sympy 1.12",
+    raises=AssertionError,
+    strict=True,
+)
+@pytest.mark.xfail(
+    condition=(SYMPY_VERSION in (Version("1.9"), Version("1.10"))),
     reason="Not resolved upstream as of sympy 1.10",
     raises=AttributeError,
     strict=True,
@@ -866,7 +867,11 @@ def test_unpickling_old_array():
     # the expected error is "AttributeError: 'One' object has no attribute '__dict__'"
     PFILE = Path(__file__).parent / "data" / "unyt_array_sympy1.8.pickle"
     with open(PFILE, "rb") as fh:
-        pickle.load(fh)
+        arr = pickle.load(fh)
+
+    # this comparison fails with sympy==1.12
+    # see https://github.com/sympy/sympy/issues/25134
+    assert arr.units.dimensions == cm.dimensions
 
 
 def test_copy():
@@ -962,7 +967,6 @@ def unary_ufunc_comparison(ufunc, a):
             "positive",
         ]
     ):
-
         ret = ufunc(a, out=out)
 
         assert_array_equal(ret, out)
@@ -1300,7 +1304,6 @@ def test_reductions():
 
 
 def test_convenience():
-
     for orig in [
         [1.0, 2.0, 3.0],
         (1.0, 2.0, 3.0),
@@ -1375,7 +1378,6 @@ def test_registry_association():
 
 
 def test_to_value():
-
     a = unyt_array([1.0, 2.0, 3.0], "kpc")
     assert_equal(a.to_value(), np.array([1.0, 2.0, 3.0]))
     assert_equal(a.to_value(), a.value)
@@ -1387,55 +1389,66 @@ def test_to_value():
 
 
 def test_astropy():
-    if isinstance(_astropy.__version__, NotAModule):
-        return
+    pytest.importorskip("astropy")
+
     ap_arr = np.arange(10) * _astropy.units.km / _astropy.units.hr
-    yt_arr = unyt_array(np.arange(10), "km/hr")
-    yt_arr2 = unyt_array.from_astropy(ap_arr)
+    un_arr = unyt_array(np.arange(10), "km/hr")
+    un_arr2 = unyt_array.from_astropy(ap_arr)
 
     ap_quan = 10.0 * _astropy.units.Msun**0.5 / (_astropy.units.kpc**3)
-    yt_quan = unyt_quantity(10.0, "sqrt(Msun)/kpc**3")
-    yt_quan2 = unyt_quantity.from_astropy(ap_quan)
+    un_quan = unyt_quantity(10.0, "sqrt(Msun)/kpc**3")
+    un_quan2 = unyt_quantity.from_astropy(ap_quan)
 
-    assert_array_equal(ap_arr, yt_arr.to_astropy())
-    assert_array_equal(yt_arr, unyt_array.from_astropy(ap_arr))
-    assert_array_equal(yt_arr, yt_arr2)
+    assert_array_equal(ap_arr, un_arr.to_astropy())
+    assert_array_equal(un_arr, unyt_array.from_astropy(ap_arr))
+    assert_array_equal(un_arr, un_arr2)
 
-    assert_equal(ap_quan, yt_quan.to_astropy())
-    assert_equal(yt_quan, unyt_quantity.from_astropy(ap_quan))
-    assert_equal(yt_quan, yt_quan2)
+    assert_equal(ap_quan, un_quan.to_astropy())
+    assert_equal(un_quan, unyt_quantity.from_astropy(ap_quan))
+    assert_equal(un_quan, un_quan2)
 
-    assert_array_equal(yt_arr, unyt_array.from_astropy(yt_arr.to_astropy()))
-    assert_equal(yt_quan, unyt_quantity.from_astropy(yt_quan.to_astropy()))
+    assert_array_equal(un_arr, unyt_array.from_astropy(un_arr.to_astropy()))
+    assert_equal(un_quan, unyt_quantity.from_astropy(un_quan.to_astropy()))
+
+
+def test_astropy_dimensionless():
+    # see https://github.com/yt-project/unyt/issues/436
+    pytest.importorskip("astropy")
+
+    arr = unyt_array([1, 2, 3], "")
+    ap_arr = np.array([1, 2, 3]) * _astropy.units.Unit("")
+
+    assert_array_equal(ap_arr, arr.to_astropy())
+    assert_array_equal(arr, unyt_array.from_astropy(ap_arr))
 
 
 def test_pint():
+    pytest.importorskip("pint")
+
     def assert_pint_array_equal(arr1, arr2):
         assert_array_equal(arr1.magnitude, arr2.magnitude)
         assert str(arr1.units) == str(arr2.units)
 
-    if isinstance(_pint.UnitRegistry, NotAModule):
-        return
     ureg = _pint.UnitRegistry()
 
     p_arr = np.arange(10) * ureg.km / ureg.year
-    yt_arr = unyt_array(np.arange(10), "km/yr")
-    yt_arr2 = unyt_array.from_pint(p_arr)
+    un_arr = unyt_array(np.arange(10), "km/yr")
+    un_arr2 = unyt_array.from_pint(p_arr)
 
     p_quan = 10.0 * ureg.g**0.5 / (ureg.mm**3)
-    yt_quan = unyt_quantity(10.0, "sqrt(g)/mm**3")
-    yt_quan2 = unyt_quantity.from_pint(p_quan)
+    un_quan = unyt_quantity(10.0, "sqrt(g)/mm**3")
+    un_quan2 = unyt_quantity.from_pint(p_quan)
 
-    assert_pint_array_equal(p_arr, yt_arr.to_pint())
-    assert_array_equal(yt_arr, unyt_array.from_pint(p_arr))
-    assert_array_equal(yt_arr, yt_arr2)
+    assert_pint_array_equal(p_arr, un_arr.to_pint())
+    assert_array_equal(un_arr, unyt_array.from_pint(p_arr))
+    assert_array_equal(un_arr, un_arr2)
 
-    assert_pint_array_equal(p_quan, yt_quan.to_pint())
-    assert_equal(yt_quan, unyt_quantity.from_pint(p_quan))
-    assert_equal(yt_quan, yt_quan2)
+    assert_pint_array_equal(p_quan, un_quan.to_pint())
+    assert_equal(un_quan, unyt_quantity.from_pint(p_quan))
+    assert_equal(un_quan, un_quan2)
 
-    assert_array_equal(yt_arr, unyt_array.from_pint(yt_arr.to_pint()))
-    assert_equal(yt_quan, unyt_quantity.from_pint(yt_quan.to_pint()))
+    assert_array_equal(un_arr, unyt_array.from_pint(un_arr.to_pint()))
+    assert_equal(un_quan, unyt_quantity.from_pint(un_quan.to_pint()))
 
 
 def test_subclass():
@@ -1443,7 +1456,7 @@ def test_subclass():
         def __new__(
             cls, input_array, units=None, registry=None, bypass_validation=None
         ):
-            return super(unyt_a_subclass, cls).__new__(
+            return super().__new__(
                 cls,
                 input_array,
                 units,
@@ -1519,8 +1532,7 @@ def test_string_operations_raise_errors_quantity():
 
 
 def test_h5_io():
-    if isinstance(_h5py.__version__, NotAModule):
-        return
+    pytest.importorskip("h5py")
 
     tmpdir = tempfile.mkdtemp()
     curdir = os.getcwd()
@@ -2042,7 +2054,7 @@ def test_electromagnetic():
     assert_almost_equal((1.0 / (u.clight**2 * mu_0)).in_units(eps_0.units), eps_0)
 
 
-def test_ytarray_coercion():
+def test_unyt_array_coercion():
     a = unyt_array([1, 2, 3], "cm")
     q = unyt_quantity(3, "cm")
     na = np.array([1, 2, 3])
@@ -2069,40 +2081,55 @@ def test_numpy_wrappers():
     arr_norm_answer = [norm_answer, np.sqrt(4**2 + 5**2 + 6**2)]
     dot_answer = 14
 
-    assert_array_equal(unyt_array(catenate_answer, "cm"), uconcatenate((a1, a2)))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unyt_array(catenate_answer, "cm"), uconcatenate((a1, a2)))
     assert_array_equal(catenate_answer, np.concatenate((a1, a2)))
 
-    assert_array_equal(unyt_array(intersect_answer, "cm"), uintersect1d(a1, a2))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unyt_array(intersect_answer, "cm"), uintersect1d(a1, a2))
     assert_array_equal(intersect_answer, np.intersect1d(a1, a2))
 
-    assert_array_equal(unyt_array(union_answer, "cm"), uunion1d(a1, a2))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unyt_array(union_answer, "cm"), uunion1d(a1, a2))
     assert_array_equal(union_answer, np.union1d(a1, a2))
 
-    assert_array_equal(
-        unyt_array(cross_answer, "cm**2"), ucross(a1, a1 + (2 * a1.units))
-    )
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(
+            unyt_array(cross_answer, "cm**2"), ucross(a1, a1 + (2 * a1.units))
+        )
     assert_array_equal(cross_answer, np.cross(a1.v, a1.v + 2))
 
-    assert_array_equal(unorm(a1), unyt_quantity(norm_answer, "cm"))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unorm(a1), unyt_quantity(norm_answer, "cm"))
     assert_array_equal(np.linalg.norm(a1), norm_answer)
-    assert_array_equal(unorm(a3, axis=1), unyt_array(arr_norm_answer, "cm"))
+
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unorm(a3, axis=1), unyt_array(arr_norm_answer, "cm"))
     assert_array_equal(np.linalg.norm(a3, axis=1), arr_norm_answer)
 
-    assert_array_equal(udot(a1, a1), unyt_quantity(dot_answer, "cm**2"))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(udot(a1, a1), unyt_quantity(dot_answer, "cm**2"))
 
-    assert_array_equal(np.array(catenate_answer), uconcatenate((a1.v, a2.v)))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(np.array(catenate_answer), uconcatenate((a1.v, a2.v)))
     with pytest.raises(RuntimeError):
-        uconcatenate((a1, a2.v))
+        with pytest.warns(DeprecationWarning):
+            uconcatenate((a1, a2.v))
     with pytest.raises(RuntimeError):
-        uconcatenate((a1.to("m"), a2))
-    assert_array_equal(unyt_array(vstack_answer, "cm"), uvstack([a2, a4]))
+        with pytest.warns(DeprecationWarning):
+            uconcatenate((a1.to("m"), a2))
+
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unyt_array(vstack_answer, "cm"), uvstack([a2, a4]))
     assert_array_equal(vstack_answer, np.vstack([a2, a4]))
-    assert_array_equal(unyt_array(vstack_answer, "cm"), ustack([a2, a4]))
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(unyt_array(vstack_answer, "cm"), ustack([a2, a4]))
     assert_array_equal(vstack_answer, np.stack([a2, a4]))
 
-    assert_array_equal(
-        unyt_array(vstack_answer_last_axis, "cm"), ustack([a2, a4], axis=-1)
-    )
+    with pytest.warns(DeprecationWarning):
+        assert_array_equal(
+            unyt_array(vstack_answer_last_axis, "cm"), ustack([a2, a4], axis=-1)
+        )
     assert_array_equal(vstack_answer_last_axis, np.stack([a2, a4], axis=-1))
 
 
@@ -2204,7 +2231,6 @@ def test_builtin_sum():
 
 
 def test_initialization_different_registries():
-
     reg1 = UnitRegistry()
     reg2 = UnitRegistry()
 
@@ -2395,26 +2421,6 @@ def test_overflow_warnings():
     _process_warning(data.convert_to_units, message, RuntimeWarning, ("mile",))
 
 
-def test_input_units_deprecation():
-    from unyt.array import unyt_array, unyt_quantity
-
-    message = "input_units has been deprecated, please use units instead"
-
-    _process_warning(
-        unyt_array, message, DeprecationWarning, ([1, 2, 3],), {"input_units": "mile"}
-    )
-    _process_warning(
-        unyt_quantity, message, DeprecationWarning, (3,), {"input_units": "mile"}
-    )
-
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        assert_array_equal(
-            unyt_array([1, 2, 3], "mile"), unyt_array([1, 2, 3], input_units="mile")
-        )
-        assert unyt_quantity(3, "mile") == unyt_quantity(3, input_units="mile")
-
-
 def test_clip():
     from unyt import km
 
@@ -2485,11 +2491,8 @@ def test_delta_degC():
     t1 = 10 * degC
     t2 = 1 * K
     assert t1 + t2 == 11 * degC
-    # In the current implementation 1*K + 10*degC = 11*K
-    # In the future, this will generate UnitOperationError
-    with pytest.warns(FutureWarning):
+    with pytest.raises(UnitOperationError):
         t2 + t1
-    assert t2 + t1 == 11 * K
     t3 = 1 * delta_degC
     assert t1 + t3 == 11 * degC
     assert t3 + t1 == 11 * degC
@@ -2501,11 +2504,8 @@ def test_delta_degF():
     t1 = 10 * degF
     t2 = 1 * R
     assert t1 + t2 == 11 * degF
-    # In the current implementation 1*R + 10*degF = 11*R
-    # In the future, this will generate UnitOperationError
-    with pytest.warns(FutureWarning):
+    with pytest.raises(UnitOperationError):
         t2 + t1
-    assert t2 + t1 == 11 * R
     t3 = 1 * delta_degF
     assert t1 + t3 == 11 * degF
     assert t3 + t1 == 11 * degF
@@ -2513,6 +2513,26 @@ def test_delta_degF():
     assert 2 * delta_degF == unyt_quantity(2, "delta_degF")
 
 
+@pytest.mark.parametrize(
+    ("u0", "u1", "uout"),
+    [
+        (K, K, K),
+        (R, R, R),
+        (degC, degC, delta_degC),
+        (degF, degF, delta_degF),
+        (degC, delta_degC, degC),
+        (delta_degC, degC, degC),
+        (degF, delta_degF, degF),
+        (delta_degF, degF, degF),
+    ],
+)
+def test_delta_temperature_diff(u0, u1, uout):
+    # using repr comparison because
+    # 1) we don't care that Unit instances might not be the same
+    # 2) some temperature units will compare as equal even when they are not identical (e.g. K and delta_degC)
+    assert repr((2 * u0 - 1 * u1).units) == repr(uout)
+
+
 def test_mil():
     assert_allclose_units(unyt_quantity(1, "mil"), unyt_quantity(0.001, "inch"))
 
@@ -2586,8 +2606,8 @@ def test_complexvalued(tmp_path):
 def test_string_formatting():
     d = unyt_array((1, 2, 3), "Msun")
     expected = "[1 2 3] Msun"
-    assert "%s" % d == expected
-    assert "{}".format(d) == expected
+    assert f"{d}" == expected
+    assert f"{d}" == expected
 
 
 @pytest.mark.parametrize(
@@ -2679,7 +2699,7 @@ def test_invalid_unit_quantity_from_string(s):
     un_str = s.split()[-1]
     with pytest.raises(
         UnitParseError,
-        match="Could not find unit symbol '{}' in the provided symbols.".format(un_str),
+        match=f"Could not find unit symbol '{un_str}' in the provided symbols.",
     ):
         unyt_quantity.from_string(s)
 
